/*
===============================================================================
 Name        : main.c
 Author      : Radan Vowles
 Copyright   : Copyright (C) 2011
 Description : Control Program for Telescope GOTO Mount
===============================================================================
*/
#define	STVersion	"v2-120606"
#define UCs			(unsigned char *)
#define STCTRL      (*(volatile unsigned long *)(0xE000E010))
#define STRELOAD    (*(volatile unsigned long *)(0xE000E014))

#ifdef __USE_CMSIS
#include "LPC17xx.h"
#endif

#include <cr_section_macros.h>
#include <NXP/crp.h>

// Variable to store CRP value in. Will be placed automatically
// by the linker when "Enable Code Read Protect" selected.
// See crp.h header for more information
__CRP const unsigned int CRP_WORD = CRP_NO_CRP ;

#include <math.h>
#include <NXP/LPC17xx/type.h>
#include <NXP/LPC17xx/LPC17xx.h>
#include "scopetrack.h"

// 5+6+5 bit colour definitions
#define WHITEH 	0b11111111
#define WHITEL	0b11111111
#define BLACKH 	0b00000000
#define BLACKL 	0b00000000
#define REDH 	0b11111000
#define REDL 	0b00000000
#define GREENH 	0b00000111
#define GREENL 	0b11100000
#define BLUEH 	0b00000000
#define BLUEL 	0b00011111

volatile double uLng, uLat, uLST, tRA, tDec, tHA, tAlt, tAz, aAlt, aAz, aRA, aDec;	// u-user, t-target, a-actual
volatile long rotor1, rotor2, rotor3, rAim1, rAim2, rAim3;		// rotor1/2/3 = current motor position, rAim1/2/3 = target for motor
volatile long rSpd1, rSpd2, rSpd3;	// Motor Speeds
unsigned char* tStarName;
volatile long msecs, uTZ, tChart;

volatile unsigned long UART3Status, UART3Count = 0, UART3IN = 0, UART3OUT;	// FIFO Receive to IN retrieve from OUT
volatile unsigned char UART3TxEmpty=1, UART3Buffer[BUFSIZE], tFollow=0, tHome = 0;
static int GR	= 200*50*10*2;	// Gear Ratio

// *******************************************************
// General Register and Peripheral Info:
// *******************************************************
// Serial IO Port:
// TX = P4:28 (TXD3)
// RX = P4:29 (RXD3)

// *******************************************************
// LCD Display: 160x160 rgb lcd
unsigned char lcd_sclk	= 0b00001000;	// SCLK : P2:0 FIO1PIN2 bit 19 - Clock
unsigned char lcd_sdat	= 0b00010000;	// SDAT : P2:1 FIO1PIN2 bit 20 - Data
unsigned char lcd_cs	= 0b00100000;	// _/CS : P2:2 FIO1PIN2 bit 21 - low to select
unsigned char lcd_dc	= 0b01000000;	// _D/C : P2:3 FIO1PIN2 bit 22 - 1=D, 0=C
unsigned char lcd_rst	= 0b10000000;	// /RST : P2:4 FIO1PIN2 bit 23 - low to reset

//*********************************************************
// Stepper 1 Motor for Azimuth Control - 1.8 / 50 / 10 / 2 = 0.0018 deg/step, 360/200000
// Stepper 1 Coil 1 : P1:24 : FIO1PIN3 bit 0
// Stepper 1 Coil 2 : P1:25 : FIO1PIN3 bit 1
// Stepper 1 Coil 3 : P1:26 : FIO1PIN3 bit 2
// Stepper 1 Coil 4 : P1:27 : FIO1PIN3 bit 3

// Stepper 2 Motor for Altitude Control - 1.8 / 50 / 10 / 2 = 0.0018 deg/step, 360/200000
// Stepper 2 Coil 1 : P0:8 : FIO0PIN1 bit 0
// Stepper 2 Coil 2 : P0:9 : FIO0PIN1 bit 1
// Stepper 2 Coil 3 : P0:10 : FIO0PIN1 bit 2
// Stepper 2 Coil 4 : P0:11 : FIO0PIN1 bit 3

// Stepper 3 Motor for Focus Control
// Stepper 3 Coil 1 : P0:24 : FIO0PIN3 bit 0
// Stepper 3 Coil 2 : P0:25 : FIO0PIN3 bit 1
// Stepper 3 Coil 3 : P0:26 : FIO0PIN3 bit 2
// Stepper 3 Coil 4 : P0:27 : FIO0PIN3 bit 3
//**********************************************************
// INPUTS FOR CONTROL AND SENSOR MICROSWITCHES
// P2:0-7 for hand-control unit:
// P2:0 - UP		- Normal mode: tAlt++, Align mode: rotor2-- (will cause up movement)
// P2:1 - DOWN		- Normal mode: tAlt--, Align mode: rotor2++
// P2:2 - LEFT		- Normal mode: tAz-- , Align mode: rotor1++
// P2:3 - RIGHT		- Normal mode: tAz++ , Align mode: rotor1--
// P2:4 - HOME		- Normal mode: tAz & tAlt = 0, Align mode: rotor1 & rotor2 = 0
// P2:5 - ENTER		- Toggle mode and display mode on LCD
// P2:6 - FOCUS IN	- rAim3--
// P2:7 - FOCUS OUT - rAim3++
// Also use P2:11-13 as home switch inputs Az/Alt/Focus respectively from micro-switches
// P2:11 - Azimuth is at home position
// P2:12 - Altitude is at home position
// P2:13 - Focus is at home position
//**********************************************************

// Delay timer uSec input
void delay(long usec) {
	long i, count;
	count = usec * 8;
	for (i=0; i<count; i++);
}

void UART3_IRQHandler (void)
{
  unsigned char IIRValue, LSRValue, Dummy = Dummy;

  IIRValue = LPC_UART3->IIR;

  IIRValue >>= 1;			/* skip pending bit in IIR */
  IIRValue &= 0x07;			/* check bit 1~3, interrupt identification */
  if ( IIRValue == IIR_RLS )		/* Receive Line Status */
  {
	LSRValue = LPC_UART3->LSR;
	/* Receive Line Status */
	if ( LSRValue & (LSR_OE|LSR_PE|LSR_FE|LSR_RXFE|LSR_BI) )
	{
	  /* There are errors or break interrupt */
	  /* Read LSR will clear the interrupt */
	  UART3Status = LSRValue;
	  Dummy = LPC_UART3->RBR;		/* Dummy read on RX to clear
							interrupt, then bail out */
	  return;
	}
	if ( LSRValue & LSR_RDR )	/* Receive Data Ready */
	{
	  /* If no error on RLS, normal ready, save into the data buffer. */
	  /* Note: read RBR will clear the interrupt */
	  UART3Buffer[UART3IN] = LPC_UART3->RBR;
	  UART3Count++;
	  UART3IN++;
	  if ( UART3Count == BUFSIZE ) UART3Count = 0;		/* buffer overflow */
	  if ( UART3IN == BUFSIZE ) UART3IN = 0;			// Loop FIFO
	}
  }
  else if ( IIRValue == IIR_RDA )	/* Receive Data Available */
  {
	/* Receive Data Available */
	  UART3Buffer[UART3IN] = LPC_UART3->RBR;
	  UART3Count++;
	  UART3IN++;
	  if ( UART3Count == BUFSIZE ) UART3Count = 0;		/* buffer overflow */
	  if ( UART3IN == BUFSIZE ) UART3IN = 0;			// Loop FIFO
  }
  else if ( IIRValue == IIR_CTI )	/* Character timeout indicator */
  {
	/* Character Time-out indicator */
	UART3Status |= 0x100;		/* Bit 9 as the CTI error */
  }
  else if ( IIRValue == IIR_THRE )	/* THRE, transmit holding register empty */
  {
	/* THRE interrupt */
	LSRValue = LPC_UART3->LSR;		/* Check status in the LSR to see if
									valid data in U0THR or not */
	if ( LSRValue & LSR_THRE )
	{
	  UART3TxEmpty = 1;
	}
	else
	{
	  UART3TxEmpty = 0;
	}
  }
}

// Send a String to UART3
void UART3PutS(unsigned char Str[]){
	int i = 0;
	while (Str[i] != 0){
		while (!UART3TxEmpty);
		U3THR = Str[i];
		UART3TxEmpty = 0;
		i++;
	}
	while (!UART3TxEmpty);
	U3THR = 13;
	UART3TxEmpty = 0;
	while (!UART3TxEmpty);
	U3THR = 10;
	UART3TxEmpty = 0;
}

// Get a single byte/chr from UART3
unsigned char UART3GetB(){
	unsigned char chr;
	if (UART3Count == 0) return(0);
	chr = UART3Buffer[UART3OUT];
	UART3Count--;
	UART3OUT++;
	if (UART3OUT == BUFSIZE) UART3OUT = 0;
	return(chr);
}

// Convert Alt/Az to RA/Dec for Target or current Aim:
// ConvH2C(x); Where x = 0 or 1 as:
// 0 = Convert stored tAlt, tAz, uLng, uLat & time to tRA & tDec
// 1 = Convert stored aAlt, aAz, uLng, uLat & time to aRA & aDec
void ConvH2C(int x){
	double cDays, TD, TG, TL, cRA, RA, DC, Al, Az, Lat;

	if (x == 0) {
		Al = tAlt;
		Az = tAz;
	} else {
		Al = aAlt;
		Az = aAz;
	}
	cDays = SEC;
	cDays /= 60;
	cDays += MIN;
	cDays /= 60;
	cDays += HOUR;
	cDays -= (uTZ/10);
	cDays /= 24;
	cDays += DOM;
	switch (MONTH){
	case 1:
		cDays += 0;
		break;
	case 2:
		cDays += 31;
		break;
	case 3:
		cDays += 59;
		break;
	case 4:
		cDays += 90;
		break;
	case 5:
		cDays += 120;
		break;
	case 6:
		cDays += 151;
		break;
	case 7:
		cDays += 181;
		break;
	case 8:
		cDays += 212;
		break;
	case 9:
		cDays += 243;
		break;
	case 10:
		cDays += 273;
		break;
	case 11:
		cDays += 304;
		break;
	case 12:
		cDays += 334;
		break;
	}
	if (!(YEAR % 4) && (MONTH > 2)) cDays++;
	switch (YEAR) {
	case 2011:
		cDays += 4016.5;
		break;
	case 2012:
		cDays += 4381.5;
		break;
	case 2013:
		cDays += 4747.5;
		break;
	case 2014:
		cDays += 5112.5;
		break;
	case 2015:
		cDays += 5477.5;
		break;
	}
	TD = cDays/36525.0;
	TG = 67310.54841 + (3164400184.812866 * TD) + (0.093104 * pow(TD, 2.0)) - (0.0000062 * pow(TD, 3.0));
	TG = fmod((fmod(TG,86400*(TG/fabs(TG)))/240),360);
	TL = TG + uLng;
	TL = fmod(TL, 360);
	// Al, Az & uLat need to be in radians for math functions
	Al  = Al   * 0.0174532925;
	Az  = Az   * 0.0174532925;
	Lat = uLat * 0.0174532925;
	DC  = asin(sin(Al)*sin(Lat) + cos(Al)*cos(Lat)*cos(Az));
	cRA = (sin(Al) - sin(DC)*sin(Lat))/(cos(DC)*cos(Lat));
	RA  = acos(cRA);
	// Convert back to degrees
	RA = RA * 57.2957795;
	DC = DC * 57.2957795;
	if (sin(Az) > 0) RA = 360 - RA;
	RA = TL - RA;
	if (RA < 0) RA += 360;
	if (DC >= 0) DC = fabs(DC);
	else DC = -fabs(DC);
	if (x == 0) {
		tRA = RA/15;
		tDec = DC;
		tStarName = UCs "            ";
	} else {
		aRA = RA/15;
		aDec = DC;
	}
}

// Wait for and retrieve new time command data as
// Chhmmss,ddmmyyyy,+tz.z	T+21 - Set Time,Date,Timezone
// Timeout if not reached 21 bytes within ~ 15secs
void GetNewTime(){
	unsigned char *msg[32];
	int dig1, dig2, hr, mn, se, da, mo, yr, tz, neg;

	*msg = UCs "Error";
	dig1 = HOUR * 3600 + MIN * 60 + SEC;
	dig2 = dig1+15;
	while ((dig1 < dig2)){
		if (UART3Count > 20){
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig1 -= 48;
			dig1 *=10;
			dig1 += (dig2-48);
			if ((dig1 < 0) | (dig1 > 23)) break;
			hr = dig1;
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig1 -= 48;
			dig1 *=10;
			dig1 += (dig2-48);
			if ((dig1 < 0) | (dig1 > 59)) break;
			mn = dig1;
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig1 -= 48;
			dig1 *=10;
			dig1 += (dig2-48);
			if ((dig1 < 0) | (dig1 > 59)) break;
			se = dig1;
			dig1 = UART3GetB();
			if (dig1 != ',') break;
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig1 -= 48;
			dig1 *=10;
			dig1 += (dig2-48);
			if ((dig1 < 1) | (dig1 > 31)) break;
			da = dig1;
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig1 -= 48;
			dig1 *=10;
			dig1 += (dig2-48);
			if ((dig1 < 1) | (dig1 > 12)) break;
			mo = dig1;
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig1 -= 48;
			dig1 *=10;
			dig1 += (dig2-48);
			if ((dig1 < 20) | (dig1 > 24)) break;
			yr = dig1*100;
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig1 -= 48;
			dig1 *=10;
			dig1 += (dig2-48);
			if ((dig1 < 0) | (dig1 > 99)) break;
			yr += dig1;
			dig1 = UART3GetB();
			if (dig1 != ',') break;
			dig1 = UART3GetB();
			if (dig1 == '+') neg = 0;
			else if (dig1 == '-') neg = 1;
			else break;
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig1 -= 48;
			dig1 *=10;
			dig1 += (dig2-48);
			if ((dig1 < 0) | (dig1 > 12)) break;
			tz = dig1;
			dig1 = UART3GetB();
			if (dig1 != '.') break;
			dig1 = UART3GetB();
			dig1 -= 48;
			if ((dig1 < 0) | (dig1 > 9)) break;
			tz *= 10;
			tz += dig1;
			HOUR = hr;
			MIN = mn;
			SEC = se;
			DOM = da;
			MONTH = mo;
			YEAR = yr;
			if (neg) uTZ = -tz;
			else uTZ = tz;
			GPREG0 = uTZ+12;
			*msg = UCs "# OK";
		} else dig1 = HOUR * 3600 + MIN * 60 + SEC;
	}
	UART3PutS(*msg);
}

// Set Current Location as:
// Pxxx.xxxEyyy.yyyS		L+16 - Set Location
// Timeout if not reached 21 bytes within ~ 15secs
void GetNewLocation(){
	unsigned char *msg[32];
	int dig1, dig2, dig3;
	double ln, la, ix;

	*msg = UCs "Error";
	dig1 = HOUR * 3600 + MIN * 60 + SEC;
	dig2 = dig1+15;
	while ((dig1 < dig2)){
		if (UART3Count > 15){
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig1 -= 48;
			dig1 *=100;
			dig1 += (dig2-48)*10;
			dig1 += (dig3-48);
			if ((dig1 < 0) | (dig1 > 179)) break;
			ln = dig1;
			dig1 = UART3GetB();
			if (dig1 != '.') break;
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig1 -= 48;
			dig1 *=100;
			dig1 += (dig2-48)*10;
			dig1 += (dig3-48);
			if ((dig1 < 0) | (dig1 > 999)) break;
			ix = dig1;
			ln += ix / 1000;
			dig1 = UART3GetB();
			if (dig1 == 'E') ln = ln;
			else if (dig1 == 'W') ln = -ln;
			else break;

			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig1 -= 48;
			dig1 *=100;
			dig1 += (dig2-48)*10;
			dig1 += (dig3-48);
			if ((dig1 < 0) | (dig1 > 179)) break;
			la = dig1;
			dig1 = UART3GetB();
			if (dig1 != '.') break;
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig1 -= 48;
			dig1 *=100;
			dig1 += (dig2-48)*10;
			dig1 += (dig3-48);
			if ((dig1 < 0) | (dig1 > 999)) break;
			ix = dig1;
			la += ix / 1000;
			dig1 = UART3GetB();
			if (dig1 == 'N') la = la;
			else if (dig1 == 'S') la = -la;
			else break;
			uLng = ln;
			uLat = la;
			*msg = UCs "# OK";
			break;
		} else dig1 = HOUR * 3600 + MIN * 60 + SEC;
	}
	UART3PutS(*msg);
}

// Set Target RA/DEC as:
// Gxx.xxxx,+yy.yyyy		G+16 - Set Goto RA/DEc
// Timeout if not reached 21 bytes within ~ 15secs
void GetNewTarget(){
	unsigned char *msg[32];
	int dig1, dig2, dig3, dig4;
	double ra, de, ix, neg;

	*msg = UCs "Error";
	dig1 = HOUR * 3600 + MIN * 60 + SEC;
	dig2 = dig1+15;
	while ((dig1 < dig2)){
		if (UART3Count > 15){
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig1 -= 48;
			dig1 *=10;
			dig1 += (dig2-48);
			if ((dig1 < 0) | (dig1 > 23)) break;
			ra = dig1;
			dig1 = UART3GetB();
			if (dig1 != '.') break;
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			dig1 *=1000;
			dig1 += (dig2-48)*100;
			dig1 += (dig3-48)*10;
			dig1 += (dig4-48);
			if ((dig1 < 0) | (dig1 > 9999)) break;
			ix = dig1;
			ra += ix / 10000;

			dig1 = UART3GetB();
			if (dig1 != ',') break;

			dig1 = UART3GetB();
			if (dig1 == '+') neg = 0;
			else if (dig1 == '-') neg = 1;
			else break;

			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig1 -= 48;
			dig1 *=10;
			dig1 += (dig2-48);
			if ((dig1 < 0) | (dig1 > 89)) break;
			de = dig1;
			dig1 = UART3GetB();
			if (dig1 != '.') break;
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			dig1 *=1000;
			dig1 += (dig2-48)*100;
			dig1 += (dig3-48)*10;
			dig1 += (dig4-48);
			if ((dig1 < 0) | (dig1 > 9999)) break;
			ix = dig1;
			de += ix / 10000;
			tRA = ra;
			if (neg) tDec = -de;
			else tDec = de;
			tStarName = UCs "custom";
			*msg = UCs "# OK";
			break;
		} else dig1 = HOUR * 3600 + MIN * 60 + SEC;
	}
	UART3PutS(*msg);
}

// Get new Altitude and Azimuth for scope and GOTO
// format : Bxxxx,yyyy = B+9 where x = Azimuth and y = Altitude
void GetNewAltAz(){
	unsigned char *msg[32];
	int dig1, dig2, dig3, dig4;
	double lAlt, lAz, ix;

	*msg = UCs "# Error";
	dig1 = HOUR * 3600 + MIN * 60 + SEC;
	dig2 = dig1+15;
	while ((dig1 < dig2)){
		if (UART3Count > 8){
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			if (dig1 > 9) dig1 -= 7;
			dig2 -= 48;
			if (dig2 > 9) dig2 -= 7;
			dig3 -= 48;
			if (dig3 > 9) dig3 -= 7;
			dig4 -= 48;
			if (dig4 > 9) dig4 -= 7;
			ix = dig1*0x1000 + dig2*0x100 + dig3*0x10 + dig1;
			lAz = (ix / 0x10000) * 360;

			dig1 = UART3GetB();
			if (dig1 != ',') lAz = -1;	// force error if syntax not correct

			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			if (dig1 > 9) dig1 -= 7;
			dig2 -= 48;
			if (dig2 > 9) dig2 -= 7;
			dig3 -= 48;
			if (dig3 > 9) dig3 -= 7;
			dig4 -= 48;
			if (dig4 > 9) dig4 -= 7;
			ix = dig1*0x1000 + dig2*0x100 + dig3*0x10 + dig1;
			lAlt = (ix / 0x10000) * 360;

			if ((lAz < 0) | (lAz > 360)) break;		// oob
			if ((lAlt < 0) | (lAlt > 90)) break;	// oob

			rAim1 = lAz * GR / 360;
			rAim2 = lAlt * GR / 360;
			tAz = lAz;
			tAlt = lAlt;
			ConvH2C(0);
			tFollow = 0;
			*msg = UCs "# OK";
			break;
		} else dig1 = HOUR * 3600 + MIN * 60 + SEC;
	}
	UART3PutS(*msg);
}

// Get new Altitude and Azimuth for scope and GOTO (32-bit)
// format : Bxxxxxxxx,yyyyyyyy = B+17 where x = Azimuth and y = Altitude
void GetNewAltAz32(){
	unsigned char *msg[32];
	int dig1, dig2, dig3, dig4;
	double lAlt, lAz, ix;

	*msg = UCs "# Error";
	dig1 = HOUR * 3600 + MIN * 60 + SEC;
	dig2 = dig1+15;
	while ((dig1 < dig2)){
		if (UART3Count > 16){
			// Azimuth
			// first 4-nibbles
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			if (dig1 > 9) dig1 -= 7;
			dig2 -= 48;
			if (dig2 > 9) dig2 -= 7;
			dig3 -= 48;
			if (dig3 > 9) dig3 -= 7;
			dig4 -= 48;
			if (dig4 > 9) dig4 -= 7;
			ix = dig1*0x1000 + dig2*0x100 + dig3*0x10 + dig1;
			// second 4-nibbles
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			if (dig1 > 9) dig1 -= 7;
			dig2 -= 48;
			if (dig2 > 9) dig2 -= 7;
			dig3 -= 48;
			if (dig3 > 9) dig3 -= 7;
			dig4 -= 48;
			if (dig4 > 9) dig4 -= 7;
			ix = ix*0x10000 + dig1*0x1000 + dig2*0x100 + dig3*0x10 + dig1;
			// store result of all 8
			lAz = (ix / 0x100000000) * 360;
			// Check Syntax
			dig1 = UART3GetB();
			if (dig1 != ',') lAz = -1;	// force error if syntax not correct
			// Altitude
			// first 4-nibbles
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			if (dig1 > 9) dig1 -= 7;
			dig2 -= 48;
			if (dig2 > 9) dig2 -= 7;
			dig3 -= 48;
			if (dig3 > 9) dig3 -= 7;
			dig4 -= 48;
			if (dig4 > 9) dig4 -= 7;
			ix = dig1*0x1000 + dig2*0x100 + dig3*0x10 + dig1;
			// second 4-nibbles
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			if (dig1 > 9) dig1 -= 7;
			dig2 -= 48;
			if (dig2 > 9) dig2 -= 7;
			dig3 -= 48;
			if (dig3 > 9) dig3 -= 7;
			dig4 -= 48;
			if (dig4 > 9) dig4 -= 7;
			ix = ix*0x10000 + dig1*0x1000 + dig2*0x100 + dig3*0x10 + dig1;
			// Store result
			lAlt = (ix / 0x100000000) * 360;
			// Check not OOB
			if ((lAz < 0) | (lAz > 360)) break;		// oob
			if ((lAlt < 0) | (lAlt > 90)) break;	// oob
			// Convert to steps for motors
			rAim1 = lAz * GR / 360;
			rAim2 = lAlt * GR / 360;
			// Load into Globals
			tAz = lAz;
			tAlt = lAlt;
			// Calculate RA/DEC for tracking
			ConvH2C(0);
			tFollow = 0;
			*msg = UCs "# OK";
			break;
		} else dig1 = HOUR * 3600 + MIN * 60 + SEC;
	}
	UART3PutS(*msg);
}

// Get new RA and Dec for scope and GOTO
// format : Rxxxx,yyyy = R+9 where x = RA and y = Dec
void GetNewRaDec(){
	unsigned char *msg[32];
	int dig1, dig2, dig3, dig4;
	double lRa, lDec, ix;

	*msg = UCs "# Error";
	dig1 = HOUR * 3600 + MIN * 60 + SEC;
	dig2 = dig1+15;
	while ((dig1 < dig2)){
		if (UART3Count > 8){
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			if (dig1 > 9) dig1 -= 7;
			dig2 -= 48;
			if (dig2 > 9) dig2 -= 7;
			dig3 -= 48;
			if (dig3 > 9) dig3 -= 7;
			dig4 -= 48;
			if (dig4 > 9) dig4 -= 7;
			ix = dig1*0x1000 + dig2*0x100 + dig3*0x10 + dig1;
			lRa = (ix / 0x10000) * 360;

			dig1 = UART3GetB();
			if (dig1 != ',') lRa = -1;	// force error if syntax not correct

			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			if (dig1 > 9) dig1 -= 7;
			dig2 -= 48;
			if (dig2 > 9) dig2 -= 7;
			dig3 -= 48;
			if (dig3 > 9) dig3 -= 7;
			dig4 -= 48;
			if (dig4 > 9) dig4 -= 7;
			ix = dig1*0x1000 + dig2*0x100 + dig3*0x10 + dig1;
			if (ix>0x8000) ix = -(0x10000-ix);
			lDec = (ix / 0x10000) * 360;

			if ((lRa < 0) | (lRa > 24)) break;		// oob
			if ((lDec < -90) | (lDec > 90)) break;	// oob

			tRA = lRa;
			tDec = lDec;
			tStarName = UCs "            ";
			tFollow = 0;
			*msg = UCs "# OK";
			break;
		} else dig1 = HOUR * 3600 + MIN * 60 + SEC;
	}
	UART3PutS(*msg);
}

// Get new 32-bit RA and Dec for scope and GOTO
// format : Rxxxxxxxx,yyyyyyyy = R+17 where x = RA and y = Dec
void GetNewRaDec32(){
	unsigned char *msg[32];
	int dig1, dig2, dig3, dig4;
	double lRa, lDec, ix;

	*msg = UCs "# Error";
	dig1 = HOUR * 3600 + MIN * 60 + SEC;
	dig2 = dig1+15;
	while ((dig1 < dig2)){
		if (UART3Count > 16){
			// get 1st 4 nibbles of RA
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			if (dig1 > 9) dig1 -= 7;
			dig2 -= 48;
			if (dig2 > 9) dig2 -= 7;
			dig3 -= 48;
			if (dig3 > 9) dig3 -= 7;
			dig4 -= 48;
			if (dig4 > 9) dig4 -= 7;
			ix = dig1*0x1000 + dig2*0x100 + dig3*0x10 + dig4;
			// get 2nd 4 nibbles of RA
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			if (dig1 > 9) dig1 -= 7;
			dig2 -= 48;
			if (dig2 > 9) dig2 -= 7;
			dig3 -= 48;
			if (dig3 > 9) dig3 -= 7;
			dig4 -= 48;
			if (dig4 > 9) dig4 -= 7;
			ix = ix*0x10000 + dig1*0x1000 + dig2*0x100 + dig3*0x10 + dig4;
			lRa = (ix / 0x100000000) * 24;

			dig1 = UART3GetB();
			if (dig1 != ',') lRa = -1;	// force error if syntax not correct
			// get 1st 4 nibbles of Dec
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			if (dig1 > 9) dig1 -= 7;
			dig2 -= 48;
			if (dig2 > 9) dig2 -= 7;
			dig3 -= 48;
			if (dig3 > 9) dig3 -= 7;
			dig4 -= 48;
			if (dig4 > 9) dig4 -= 7;
			ix = dig1*0x1000 + dig2*0x100 + dig3*0x10 + dig4;
			// get 2nd 4 nibbles of Dec
			dig1 = UART3GetB();
			dig2 = UART3GetB();
			dig3 = UART3GetB();
			dig4 = UART3GetB();
			dig1 -= 48;
			if (dig1 > 9) dig1 -= 7;
			dig2 -= 48;
			if (dig2 > 9) dig2 -= 7;
			dig3 -= 48;
			if (dig3 > 9) dig3 -= 7;
			dig4 -= 48;
			if (dig4 > 9) dig4 -= 7;
			ix = ix*0x10000 + dig1*0x1000 + dig2*0x100 + dig3*0x10 + dig4;
			if (ix>0x80000000) ix = -(0x100000000-ix);
			lDec = (ix / 0x100000000) * 360;

			if ((lRa < 0) | (lRa > 24)) break;		// oob
			if ((lDec < -90) | (lDec > 90)) break;	// oob

			tRA = lRa;
			tDec = lDec;
			tStarName = UCs "            ";
			tFollow = 0;
			*msg = UCs "# OK";
			break;
		} else dig1 = HOUR * 3600 + MIN * 60 + SEC;
	}
	UART3PutS(*msg);
}

// Get Scope's current Aim as RA Dec and return via UART
// Returns as xxxxxxxx,yyyyyyyy# where x is RA, y is Dec
// Angles are in 32-bit format = (x/360)*2^32
void SendRaDec32() {
	double lRa, lDec;
	long dec;
	long ra;
	unsigned char a[20], i, n;

	ConvH2C(1);
	lRa  = (aRA / 24) * 0x100000000;
	lDec = (aDec  / 360) * 0x100000000;
	ra = lRa;
	dec = lDec;
	if (dec<0) dec += 0x100000000;
	// convert lRa to a string of 8 hex digits
	for (i = 0; i < 8; i++) {
		n = (ra >> (4*(7-i))) & 0xF;
		if (n<10) n += 48;		// 0 to '0' etc;
		else if (n<16) n += 55;	// 10 to 'A' etc;
		else n=48;
		a[i] = n;
	}
	a[8] = ',';
	// convert lDec to a string of 8 hex digits
	for (i = 0; i < 8; i++) {
		n = (dec >> (4*(7-i))) & 0xF;
		if (n<10) n += 48;	// 0 to '0' etc;
		else if (n<16) n += 55;	// 10 to 'A' etc;
		else n=48;
		a[i+9] = n;
	}
	a[17] = '#';
	a[18] = 0;
	UART3PutS(UCs a);
}

// Get Scope's current Aim as RA Dec and return via UART
// Returns as xxxx,yyyy# where x is RA, y is Dec
// Angles are in 16-bit format = (x/360)*2^16
void SendRaDec16() {
	double lRa, lDec;
	long dec;
	long ra;
	unsigned char a[20], i, n;

	ConvH2C(1);
	lRa  = (aRA / 24) * 0x10000;
	lDec = (aDec  / 360) * 0x10000;
	ra = lRa;
	dec = lDec;
	if (dec<0) dec += 0x10000;
	// convert lRa to a string of 4 hex digits
	for (i = 0; i < 4; i++) {
		n = (ra >> (4*(3-i))) & 0xF;
		if (n<10) n += 48;		// 0 to '0' etc;
		else if (n<16) n += 55;	// 10 to 'A' etc;
		else n=48;
		a[i] = n;
	}
	a[4] = ',';
	// convert lDec to a string of 4 hex digits
	for (i = 0; i < 4; i++) {
		n = (dec >> (4*(3-i))) & 0xF;
		if (n<10) n += 48;	// 0 to '0' etc;
		else if (n<16) n += 55;	// 10 to 'A' etc;
		else n=48;
		a[i+5] = n;
	}
	a[9] = '#';
	a[10] = 0;
	UART3PutS(UCs a);
}

void LcdDat(unsigned char dat1, unsigned char dat2) {
	int i;

	FIO1CLR2 = lcd_sclk;
	FIO1SET2 = lcd_dc;
	for (i=0; i<8; i++){
		if (0x80 & dat1) FIO1SET2 = lcd_sdat;
		else FIO1CLR2 = lcd_sdat;
		dat1 <<= 1;
		delay(1);
		FIO1SET2 = lcd_sclk;
		delay(1);
		FIO1CLR2 = lcd_sclk;
		delay(1);
	}
	delay(5);
	for (i=0; i<8; i++){
		if (0x80 & dat2) FIO1SET2 = lcd_sdat;
		else FIO1CLR2 = lcd_sdat;
		dat2 <<= 1;
		delay(1);
		FIO1SET2 = lcd_sclk;
		delay(1);
		FIO1CLR2 = lcd_sclk;
		delay(1);
	}
	delay(5);
}

void LcdCom(unsigned char cmd) {
	int i;

	FIO1CLR2 = lcd_sclk;
	FIO1CLR2 = lcd_dc;
	for (i=0; i<8; i++){
		if (0x80 & cmd) FIO1SET2 = lcd_sdat;
		else FIO1CLR2 = lcd_sdat;
		cmd <<= 1;
		delay(1);
		FIO1SET2 = lcd_sclk;
		delay(1);
		FIO1CLR2 = lcd_sclk;
		delay(1);
	}
	delay(5);
}

void LcdCmd(unsigned char cmd, unsigned char dat1, unsigned char dat2) {
	int i;

	FIO1CLR2 = lcd_sclk;
	FIO1CLR2 = lcd_dc;
	for (i=0; i<8; i++){
		if (0x80 & cmd) FIO1SET2 = lcd_sdat;
		else FIO1CLR2 = lcd_sdat;
		cmd <<= 1;
		delay(1);
		FIO1SET2 = lcd_sclk;
		delay(1);
		FIO1CLR2 = lcd_sclk;
		delay(1);
	}
	delay(5);
	LcdDat(dat1, dat2);
}

//******************************************************
// LcdChr(char Ascii, int x pos, int y pos)
// Prints an Ascii chr where x,y is the top left
// Start position is in char spaces (x,y) = (0,0)-(15,12)
// -----------------------------------------------------
void LcdChr(unsigned char chr, int x, int y){
	unsigned char ex, ey, adrh, adrl;
	int i1, i2, adr;
	if (x > 15) x = 15;
	if (y > 12) y = 12;
	x *= 8;
	x += 3;
	y *= 10;
	y += 2;
	ex = x + 7;
	ey = y + 9;
	chr -= 32;
	adr = x + (y * 256);
	adrl = adr % 0x100;
	adrh = adr / 0x100;
	LcdCmd(0x44, ex, x);
	LcdCmd(0x45, ey, y);
	LcdCmd(0x21, adrh, adrl);
	LcdCom(0x22);
	for (i1=0; i1 < 10; i1++){
		for (i2=0; i2 < 8; i2++){
			if (font[chr][i1] & (0x80 >> i2) ) LcdDat(REDH, REDL);
			else LcdDat(0,0);
		}
	}
}
// Print a preset message on a line
void LcdMsg(int Msg, int Ln){
	int i=0;
	while (msg[Msg][i]) {
		LcdChr(msg[Msg][i], i, Ln);
		i++;
	}
}

// LcdStr(Str, Line)
// Print a string (char array) on a line
void LcdStr(unsigned char Str[], int Ln){
	int i;

	i = 0;
	while (Str[i] != 0) {
		LcdChr(Str[i], i, Ln);
		i++;
		if (i==16) break;
	}
	while (i<16) {
		LcdChr(32,i,Ln);
		i++;
	}
}

// print a Integer number on line Ln, pos Ps in format +#####
void LcdINumber(double num, unsigned char Ln, unsigned char Ps){
	int dig;
	dig = num;
	if (dig<0){
		dig = -dig;
		LcdChr('-', Ps+0, Ln);
	} else {
		LcdChr(' ', Ps+0, Ln);
	}
	LcdChr((dig%10)+48, Ps+5, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, Ps+4, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, Ps+3, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, Ps+2, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, Ps+1, Ln);
}

void LcdClr() {
	int i;
	// Blank LCD
	LcdCom(0x22);
	for (i=0; i<17424; i++) LcdDat(0,0);
	// Print Labels
	LcdMsg(MsgTitle ,0);
	LcdMsg(MsgLTime ,1);
	LcdINumber(uTZ,3,10);
	LcdMsg(MsgLoc ,3);
	LcdStr(tStarName,5);
	LcdMsg(MsgTAim ,7);
	LcdMsg(MsgGoto ,9);
	if (tFollow) {
		LcdStr(UCs "Scope Az/Alt:On " ,11);
	} else {
		LcdStr(UCs "Scope Az/Alt:Off", 11);
	}
	LcdCmd(0x44, 131, 0);
	LcdCmd(0x45, 131, 0);
	LcdCmd(0x21, 0x0A, 0x00);
	LcdCom(0x22);
	for (i=0; i<132; i++) LcdDat(REDH, REDL);
}

void LcdTime(unsigned char Ln){
	int hr, mn, sc, da, mt, yr;
	// Print current time
	hr = HOUR;
	mn = MIN;
	sc = SEC;
	da = DOM;
	mt = MONTH;
	yr = YEAR % 100;
	LcdChr((hr/10)+48, 0, Ln);
	LcdChr((hr%10)+48, 1, Ln);
	LcdChr(':', 2, Ln);
	LcdChr((mn/10)+48, 3, Ln);
	LcdChr((mn%10)+48, 4, Ln);
	LcdChr(':', 5, Ln);
	LcdChr((sc/10)+48, 6, Ln);
	LcdChr((sc%10)+48, 7, Ln);
	LcdChr('-', 8, Ln);
	LcdChr((da/10)+48, 9, Ln);
	LcdChr((da%10)+48, 10, Ln);
//	LcdChr('/', 11, Ln);
	LcdChr((mt/10)+48, 11, Ln);
	LcdChr((mt%10)+48, 12, Ln);
//
	LcdChr((yr/10)+48, 13, Ln);
	LcdChr((yr%10)+48, 14, Ln);
}

void LcdUserLoc (unsigned char Ln) {
	int dig;

	// Print User Long/Lat
	dig = uLng*100;
	if (dig < 0) {
		LcdChr('W',6,Ln);
		dig = -dig;
	} else {
		LcdChr('E',6,Ln);
	}
	LcdChr((dig%10)+48, 5, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 4, Ln);
	LcdChr('.', 3, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 2, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 1, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 0, Ln);
	dig = uLat*100;
	if (dig < 0) {
		LcdChr('S',13,Ln);
		dig = -dig;
	} else {
		LcdChr('N',13,Ln);
	}
	LcdChr((dig%10)+48, 12, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 11, Ln);
	LcdChr('.', 10, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 9, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 8, Ln);
}

// Print Target RA/Dec on line Ln
void LcdTLoc(unsigned char Ln) {
	int dig;

	// Right Ascension as ##.###
	dig = tRA*1000;
	if (dig < 0) {
		LcdChr('-',0,Ln);
		dig = -dig;
	} else {
		LcdChr(' ',0,Ln);
	}
	LcdChr((dig%10)+48, 6, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 5, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 4, Ln);
	LcdChr('.', 3, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 2, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 1, Ln);
	// Declination as +##.###
	dig = tDec*1000;
	if (dig < 0) {
		LcdChr('-',8,Ln);
		dig = -dig;
	} else {
		LcdChr(' ',8,Ln);
	}
	LcdChr((dig%10)+48, 14, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 13, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 12, Ln);
	LcdChr('.', 11, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 10, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 9, Ln);
}

// Print Target Az/Alt on line Ln
void LcdAltAz(unsigned char Ln){
	int dig;
	dig = tAz * 1000;
	if (dig<0){
		dig = -dig;
		LcdChr('-', 0, Ln);
	} else {
		LcdChr(' ', 0, Ln);
	}
	LcdChr((dig%10)+48, 7, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 6, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 5, Ln);
	LcdChr('.', 4, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 3, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 2, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 1, Ln);

	dig = tAlt * 1000;
	if (dig<0){
		dig = -dig;
		LcdChr('-', 8, Ln);
	} else {
		LcdChr(' ', 8, Ln);
	}
	LcdChr((dig%10)+48, 15, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 14, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 13, Ln);
	LcdChr('.', 12, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 11, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 10, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 9, Ln);
}

// Print Goto Az/Alt on line Ln
void LcdGoto(unsigned char Ln){
	int dig;
	dig = rAim1 * 360 / (GR/1000);
	if (dig<0){
		dig = -dig;
		LcdChr('-', 0, Ln);
	} else {
		LcdChr(' ', 0, Ln);
	}
	LcdChr((dig%10)+48, 7, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 6, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 5, Ln);
	LcdChr('.', 4, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 3, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 2, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 1, Ln);

	dig = rAim2 * 360 / (GR/1000);
	if (dig<0){
		dig = -dig;
		LcdChr('-', 8, Ln);
	} else {
		LcdChr(' ', 8, Ln);
	}
	LcdChr((dig%10)+48, 15, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 14, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 13, Ln);
	LcdChr('.', 12, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 11, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 10, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 9, Ln);
}

// Print Current Aim Alt/Az
void LcdCAim(Ln) {
	int dig;
	dig = aAz * 1000;
	if (dig<0){
		dig = -dig;
		LcdChr('-', 0, Ln);
	} else {
		LcdChr(' ', 0, Ln);
	}
	LcdChr((dig%10)+48, 7, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 6, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 5, Ln);
	LcdChr('.', 4, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 3, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 2, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 1, Ln);

	dig = aAlt * 1000;
	if (dig<0){
		dig = -dig;
		LcdChr('-', 8, Ln);
	} else {
		LcdChr(' ', 8, Ln);
	}
	LcdChr((dig%10)+48, 15, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 14, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 13, Ln);
	LcdChr('.', 12, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 11, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 10, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 9, Ln);
}

// print a float number on line Ln in format ####.#####
void LcdNumber(double num, unsigned char Ln){
	int dig;
	dig = num * 100000;
	if (dig<0){
		dig = -dig;
		LcdChr('-', 0, Ln);
	} else {
		LcdChr(' ', 0, Ln);
	}
	LcdChr((dig%10)+48, 10, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 9, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 8, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 7, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 6, Ln);
	LcdChr('.', 5, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 4, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 3, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 2, Ln);
	dig /= 10;
	LcdChr((dig%10)+48, 1, Ln);
}

void CalcAltAz(){
	int hr, mn, se, mt, da, yr, ms;
	double cAlt, cLat, cAz, cDec, cDays, cHA;

	// first calc days since 12pm 1st Jan 2000
	hr = HOUR;
	mn = MIN;
	se = SEC;
	ms = msecs;
	da = DOM;
	mt = MONTH;
	yr = YEAR;

	cDays = ms;
	cDays /= 1000;
	cDays += se;
	cDays /= 60;
	cDays += mn;
	cDays /= 60;
	cDays += hr;
	cDays -= (uTZ/10);
	cDays /= 24;
	cDays += da;
	switch (mt){
	case 1:
		cDays += 0;
		break;
	case 2:
		cDays += 31;
		break;
	case 3:
		cDays += 59;
		break;
	case 4:
		cDays += 90;
		break;
	case 5:
		cDays += 120;
		break;
	case 6:
		cDays += 151;
		break;
	case 7:
		cDays += 181;
		break;
	case 8:
		cDays += 212;
		break;
	case 9:
		cDays += 243;
		break;
	case 10:
		cDays += 273;
		break;
	case 11:
		cDays += 304;
		break;
	case 12:
		cDays += 334;
		break;
	}
	if (!(yr % 4) && (mt > 2)) cDays++;
	switch (yr) {
	case 2011:
		cDays += 4016.5;
		break;
	case 2012:
		cDays += 4381.5;
		break;
	case 2013:
		cDays += 4747.5;
		break;
	case 2014:
		cDays += 5112.5;
		break;
	case 2015:
		cDays += 5477.5;
		break;
	}
//	LcdNumber(cDays, 11);

	// Calc Local Siderial Time
	uLST = 18.697374558 + 24.06570982441908 * cDays;
	uLST += uLng / 15;
	uLST = fmod(uLST, 24);
//	LcdNumber(uLST, 12);

	// Calc Hour Angle
	tHA = (uLST * 15) - (tRA * 15);
	if (tHA < 0) tHA += 360;
//	LcdNumber(tHA, 12);

	// Now to calc Alt/Az
	cDec 	= tDec * 0.0174532925;
	cLat 	= uLat * 0.0174532925;
	cHA 	= tHA  * 0.0174532925;
	// sin(ALT) = sin(DEC)*sin(LAT)+cos(DEC)*cos(LAT)*cos(HA)
	cAlt = (sin(cDec) * sin(cLat)) + (cos(cDec) * cos(cLat) * cos(cHA));
	cAlt = asin(cAlt);
	// cos(A) = (sin(DEC) - sin(ALT)*sin(LAT))/(cos(ALT)*cos(LAT))
	cAz = (sin(cDec) - (sin(cAlt) * sin(cLat))) / (cos(cAlt) * cos(cLat));
	cAz = acos(cAz);
	cAz *= 57.2957795;
	if (sin(cHA) < 0) tAz = cAz;
	else tAz = 360 - cAz;
	tAlt = cAlt * 57.2957795;

}

// Timer for Motor #1 (Azimuth) on Timer 0
void TIMER0_IRQHandler(void) {
		int dig, outp = 0;
		T0IR = 0b01;	// reset interrupt
		if (rotor1 < rAim1) rotor1++;
		else if (rotor1 > rAim1) rotor1--;
		else if (!tFollow) {
			// FIO1PIN3 = 0;
			T0TCR = 0b01;		// restart timer
			return;
		}
		dig = rotor1 & 0b111;
		switch (dig) {
		case 0:
			outp = 0b0001;
			break;
		case 1:
			outp = 0b0011;
			break;
		case 2:
			outp = 0b0010;
			break;
		case 3:
			outp = 0b0110;
			break;
		case 4:
			outp = 0b0100;
			break;
		case 5:
			outp = 0b1100;
			break;
		case 6:
			outp = 0b1000;
			break;
		case 7:
			outp = 0b1001;
			break;
		}
		FIO1PIN3 = outp;
		T0MR0 = rSpd1;
		T0TC  = 0;
		T0TCR = 0b01;		// restart timer
}

// Timer for Motor #2 (Altitude) on Timer 1
void TIMER1_IRQHandler(void) {
		int dig, outp = 0;
		T1IR = 0b01;	// reset interrupt
		if (rotor2 < rAim2) rotor2++;
		else if (rotor2 > rAim2) rotor2--;
		else if (!tFollow) {
			// FIO0PIN1 = 0;
			T1TCR = 0b01;		// restart timer
			return;
		}
		dig = rotor2 & 0b111;
		switch (dig) {
		case 0:
			outp = 0b0001;
			break;
		case 1:
			outp = 0b0011;
			break;
		case 2:
			outp = 0b0010;
			break;
		case 3:
			outp = 0b0110;
			break;
		case 4:
			outp = 0b0100;
			break;
		case 5:
			outp = 0b1100;
			break;
		case 6:
			outp = 0b1000;
			break;
		case 7:
			outp = 0b1001;
			break;
		}
		FIO0PIN1 = outp;
		T1MR0 = rSpd2;
		T1TC  = 0;
		T1TCR = 0b01;		// restart timer
}

// Timer for Motor #3 (Focus) on Timer 2
void TIMER2_IRQHandler(void) {
		int dig, outp = 0;
		T2IR = 0b01;	// reset interrupt
		if (rotor3 < rAim3) rotor3++;
		else if (rotor3 > rAim3) rotor3--;
		else {
			// FIO0PIN0 = 0;
			T2TCR = 0b01;		// restart timer
			return;
		}
		dig = rotor3 & 0b11;
		switch (dig) {
		case 0:
			outp = 0b0001;
			break;
		case 1:
			outp = 0b0010;
			break;
		case 2:
			outp = 0b0100;
			break;
		case 3:
			outp = 0b1000;
			break;
		}
		FIO0PIN0 = outp<<4;
		T2MR0 = rSpd3;
		T2TC  = 0;
		T2TCR = 0b01;		// restart timer
}

void SysTick_Handler(void){
	msecs+=10;
}

void RTC_IRQHandler(void){
	if (ILR & 1) msecs = 0;
	ILR = 1;
}

void init() {
	// Setup Chip Oscillator to 120MHz  (2 * 12MHz * 15) / 1 = 360MHz / 3 = 120MHz
	if (PLL0STAT & 0x2000000) {		// if PLL0 is connected then
		PLL0CON &= 0b10;			// disconnect it
		PLL0FEED = 0xAA;
		PLL0FEED = 0x55;
		PLL0CON = 0b00;				// and disable it
		PLL0FEED = 0xAA;
		PLL0FEED = 0x55;
	}
	CLKSRCSEL = 0b01;				// Select External clock source
	PLL0CFG = 0x0E;					// Select PLL0 Multipliers/Dividers = 15 / 1
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
	PLL0CON = 0b01;					// Connect PLL0
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
	CCLKCFG = 0b10;					// Select CPU Clock divider as PLL0 / 3
	while (!(PLL0STAT & 0x4000000));// Wait for PLL0 to lock
	PLL0CON = 0b11;					// Connect PLL0
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;

	// Setup RTC
	// Enable CLOCK into RTC
	PCONP |= (1 << 9);
	// If RTC is stopped, clear STOP bit.
	if ( LPC_RTC->RTC_AUX & (0x1<<4) )
	{
		LPC_RTC->RTC_AUX |= (0x1<<4);
	}
	// -- Initialise RTC registers --
	AMR = 0;
	CIIR = 0;
	CCR = 0;
	if ((YEAR < 2010) | (YEAR > 2020)) {
		HOUR = 23;
		MIN = 23;
		SEC = 0;
		DOM = 23;
		MONTH = 6;
		YEAR = 2011;
	}
	// Start the RTC
	CCR |= 0x01;
	ILR = 0x01;

	// Init the SysTick Timer
	int i = SEC;
	while (i==SEC);	// wait for RTC to tick
	msecs = 0;
	STCTRL = 7;		// Enable Timer, Interrupt & set cclk as source
	STRELOAD = 1199999;	// 10 millisec clock @ 120MHz CPU Clk
	NVIC_EnableIRQ(RTC_IRQn);
	CIIR = 1;

	// Setup ports
	FIO1DIR2 	= 0b11111100;	// LCD port all outputs
	FIO1MASK2 	= 0b00000011;	// LCD port bits 18-23 enabled
	FIO1DIR3	= 0b00001111;	// Motor 1 port all outputs
	FIO1MASK3	= 0b11110000;	// 4 bits enabled : 0-3
	FIO0DIR1	= 0b00001111;	// Motor 2 port all outputs
	FIO0MASK1	= 0b11110000;	// 4 bits enabled : 8-11
	FIO0DIR2 	= 0b01000000;	// set P0:22 Port output.
	FIO0MASK2 	= 0b10111111; 	// enable P0:22 pin for LED.
	FIO0DIR0	= 0b11110000;	// Motor 3 port all outputs
	FIO0MASK0 	= 0b00001111;	// 4 bits enabled : P0:4-7

	PINSEL4		= 0b00000000000000000000000000000000;	// P2:0-15 all as GPIO
	PINMODE4	= 0b00001111111100111111111111111111;	// define used P2 bits as Pull-Down
	FIO2DIR0	= 0b00000000;	// Hand-unit Switch Inputs
	FIO2MASK0	= 0b00000000;	// All 8 bits enabled
	FIO2DIR1	= 0b00000010;	// Home Switches
	FIO2MASK1	= 0b11000010;	//  "      "

	// Initialise LCD Module
	FIO1CLR2 = 0b11111111;
	FIO1SET2 = lcd_cs;
	delay(20000);
	FIO1SET2 = lcd_rst;
	delay(20000);
	FIO1CLR2 = lcd_cs;

	LcdCmd(0x10, 0x2f, 0x8e);	//
	delay(500);
	LcdCmd(0x11, 0x00, 0x0c);	//
	delay(500);
	LcdCmd(0x07, 0x00, 0x21);	//
	delay(500);
	LcdCmd(0x28, 0x00, 0x06);	//
	delay(500);
	LcdCmd(0x28, 0x00, 0x05);	//
	delay(500);
	LcdCmd(0x27, 0x05, 0x7f);	//
	delay(500);
	LcdCmd(0x29, 0x89, 0xa1);	//
	delay(500);
	LcdCmd(0x00, 0x00, 0x01);	//
	delay(20000);
	LcdCmd(0x29, 0x80, 0xb0);	//
	delay(2000);
	LcdCmd(0x29, 0xff, 0xfe);	//
	delay(500);
	LcdCmd(0x07, 0x00, 0x23);	//
	delay(500);
	LcdCmd(0x07, 0x00, 0x33);	//
	delay(500);
	LcdCmd(0x01, 0x21, 0x83);	//
	delay(500);
	LcdCmd(0x2f, 0xff, 0xff);	//
	delay(500);
	LcdCmd(0x2c, 0x80, 0x00);	//
	delay(500);
	LcdCmd(0x03, 0x68, 0x30);	//
	delay(500);
	LcdCmd(0x27, 0x05, 0x70);	//
	delay(500);
	LcdCmd(0x02, 0x03, 0x00);	//
	delay(500);
	LcdCmd(0x0b, 0x58, 0x0c);	//
	delay(500);
	LcdCmd(0x12, 0x06, 0x09);	//
	delay(500);
	LcdCmd(0x13, 0x31, 0x00);	//
	delay(500);
	LcdCmd(0x2a, 0x1d, 0xd0);	//
	delay(500);
	LcdCmd(0x2b, 0x0a, 0x90);	//
	delay(500);
	LcdCmd(0x2d, 0x31, 0x0f);	//
	delay(2000);
	LcdCmd(0x45, 0x83, 0x00);	//
	delay(500);
	LcdCmd(0x44, 0x83, 0x00);	//
	delay(500);
	LcdCmd(0x21, 0x00, 0x00);	//
	delay(500);
	LcdCmd(0x1e, 0x00, 0xbf);	//
	delay(500);
	LcdCmd(0x1e, 0x00, 0x00);	//
	delay(500);
	LcdCmd(0x45, 0x83, 0x00);	//
	delay(500);
	LcdCmd(0x44, 0x83, 0x00);	//
	delay(500);
	LcdCmd(0x21, 0x00, 0x00);	//
	delay(500);

	// Initialize UART with interrupt for receive.
	unsigned long Fdiv;
	PINSEL9 = 0xF000000;	// enable pins P4:28+29 for UART3
	PCONP |= 1<<25;			// Power on Uart3
	LPC_UART3->LCR = 0x83;						// 8 bits, no Parity, 1 Stop bit
	Fdiv = ((30000000 / 16) / 9600);		//baud rate
	LPC_UART3->DLM = Fdiv / 256;
	LPC_UART3->DLL = Fdiv % 256;
	LPC_UART3->LCR = 0x03;						// DLAB = 0
	LPC_UART3->FCR = 0x07;						// Enable and reset TX and RX FIFO.
	NVIC_EnableIRQ(UART3_IRQn);
	LPC_UART3->IER = 0b0111;	// Enable UART3 interrupt

	rSpd1 = 800;
	rSpd2 = 800;
	rSpd3 = 5000;
	// CCLK = 120MHz, PCLK = CCLK/4 = 30MHz
	// 5ms tick = 200Hz = 150000 ticks = 30*5000
	T0TCR = 0b10;	// Enable and Reset Timer 0 for Motor #1 (Azimuth)
	T0PR = 30;		// Timer will now click over every microsecond
	T0MR0 = rSpd1;	// every rSpdx microseconds trigger IRQ
	T0MCR = 0b111;
	NVIC_EnableIRQ(TIMER0_IRQn);
	T1TCR = 0b10;	// Enable and Reset Timer 1 for Motor #2 (Altitude)
	T1PR = 30;		// Timer will now click over every microsecond
	T1MR0 = rSpd2;	// every rSpdx microseconds trigger IRQ
	T1MCR = 0b111;
	NVIC_EnableIRQ(TIMER1_IRQn);
	PCONP |= (1<<22);// Enable power to Timer 2
	T2TCR = 0b10;	// Enable and Reset Timer 2 for Motor #3 (Focus)
	T2PR = 30;		// Timer will now click over every microsecond
	T2MR0 = rSpd3;	// every rSpdx microseconds trigger IRQ
	T2MCR = 0b111;
	NVIC_EnableIRQ(TIMER2_IRQn);

}

int main(void) {
	unsigned char op1, msg[15];
	int i,ix;
	
	init();

	// Load Defaults
	uLng = 150.6;
	uLat = -34.9;
	uTZ = GPREG0-12;

	// Load star from Chart[]
	tChart = 1;
	tStarName = Chart[tChart];
	tRA = ChartP[tChart][0];
	tDec = ChartP[tChart][1];
	//tStarName = UCs "testing";

	LcdClr();

	i = 0;
	ix = 0;
	op1 = 32;
	tHome = 1;
	// Set all motors as HOME
	// TODO : do a physical move to home position using home switches.
	rotor1 = 0;
	rotor2 = 0;
	rotor3 = 0;
	rAim1 = 0;
	rAim2 = 0;
	rAim3 = 0;

	// Print Defaults to Lcd
	LcdUserLoc(4);
	LcdTLoc(6);

	UART3PutS(UCs "OK");
	FIO0SET2 = 0b01000000;
	T0TCR = 0b01;	// Enable Timer 0
	T1TCR = 0b01;	// Enable Timer 1
	T2TCR = 0b01;	// Enable Timer 2

	// Enter an infinite loop
	while(1) {
		// LcdChr(i+48, 15, 0);
		// LcdINumber(msecs, 0, 10);
		i++;
		i %= 10;
		LcdTime(2);
		CalcAltAz();
		LcdGoto(10);
		LcdAltAz(8);
		if (tFollow) {
			rAim2 = tAlt * GR / 360;
			rAim1 = tAz * GR / 360;
			LcdStr(UCs "Scope Az/Alt:On " ,11);
		} else {
			LcdStr(UCs "Scope Az/Alt:Off" ,11);
		}
		if (tHome) {
			LcdStr(UCs "Tracking Home!", 0);
		}
		aAlt = rotor2;
		aAlt *= 360;
		aAlt /= GR;
		aAz = rotor1;
		aAz *= 360;
		aAz /= GR;
		LcdCAim(12);

		// vary motor speeds by distance
		// Motor 1 - Azimuth
/*		if (rotor1<rAim1) ix = rAim1 - rotor1;
		else ix = rotor1 - rAim1;
		if (ix > (GR/180)) {
			rSpd1 -= 100;
			if (rSpd1 < 800) rSpd1 = 4000;		// Min pulse = 1ms
		} else {
			rSpd1 += 100;
			if (rSpd1 > 4000) rSpd1 = 4000;	// Max pulse = 10ms
		}
		// Motor 2 - Altitude
		if (rotor2<rAim2) ix = rAim2 - rotor2;
		else ix = rotor2 - rAim2;
		if (ix > (GR/180)) {
			rSpd2 -= 100;
			if (rSpd2 < 800) rSpd2 = 4000;		// Min pulse = 1ms
		} else {
			rSpd2 += 100;
			if (rSpd2 > 4000) rSpd2 = 4000;	// Max pulse = 10ms
		}*/
		// LcdINumber(rotor3, 0, 0);
		// LcdINumber(rAim3, 0, 6);
		// Motor 3 - Focus
		if (rotor3<rAim3) ix = rAim2 - rotor3;
		else ix = rotor3 - rAim3;
		if (ix > (GR/180)) {
			rSpd3 -= 100;
			if (rSpd3 < 800) rSpd3 = 800;		// Min pulse = 1ms
		} else {
			rSpd3 += 100;
			if (rSpd3 > 8000) rSpd3 = 4000;	// Max pulse = 10ms
		}
		// check and execute any serial port commands
		if (UART3Count) {
			op1 = UART3GetB();
			if (op1 == 'C') {			// Custom set Clock command
				GetNewTime();
			}
			else if (op1 == 'P') {		// Custom set Position command
				GetNewLocation();
				LcdUserLoc(4);
			}
			else if (op1 == 'G') {		// Custom set tarGet command
				GetNewTarget();
				LcdTLoc(6);
			}
			else if (op1 == 'B') {		// SynScan 16-bit Goto Azm,Alt Command
				GetNewAltAz();
				rAim2 = tAlt * GR / 360;
				rAim1 = tAz * GR / 360;
				LcdTLoc(6);
				tChart = 0;
				tStarName = Chart[tChart];
				LcdStr(tStarName,5);
			}
			else if (op1 == 'B') {		// SynScan 32-bit Goto Azm,Alt Command
				GetNewAltAz32();
				rAim2 = tAlt * GR / 360;
				rAim1 = tAz * GR / 360;
				LcdTLoc(6);
				tChart = 0;
				tStarName = Chart[tChart];
				LcdStr(tStarName,5);
			}
			else if (op1 == 'R') {		// SynScan 16-bit Goto RA,Dec Command
				GetNewRaDec();
				CalcAltAz();
				rAim2 = tAlt * GR / 360;
				rAim1 = tAz * GR / 360;
				LcdTLoc(6);
				tChart = 0;
				tStarName = Chart[tChart];
				LcdStr(tStarName,5);
			}
			else if (op1 == 'r') {		// SynScan 32-bit GOTO RA-Dec
				GetNewRaDec32();
				CalcAltAz();
				rAim2 = tAlt * GR / 360;
				rAim1 = tAz * GR / 360;
				LcdTLoc(6);
				tChart = 0;
				tStarName = Chart[tChart];
				LcdStr(tStarName,5);
			}
			else if (op1 == 'e') {		// e - 32-bit GET current aim as RA-Dec
				SendRaDec32();
			}
			else if (op1 == 'E') {		// E - 16-bit GET current aim as RA-Dec
				SendRaDec16();
			}
			else if (op1 == 'T') {		// SysScan Tracking on/off Command
				while (UART3Count == 0);
				op1 = UART3GetB();
				if (op1 == '1') {
					tFollow = 1;
					UART3PutS(UCs "#");
				} else {
					tFollow = 0;
					UART3PutS(UCs "#");
				}
			}
			else if (op1 == 'H') {		// E - 16-bit GET current aim as RA-Dec
				tHome = 1;
				tFollow = 0;
				rAim1 = 0;
				rAim2 = 0;
				tAz   = 0;
				tAlt  = 0;
				UART3PutS(UCs "#");
			}
			else if (op1 == 'K') {		// > K - Echo Command - Kx returns x#"
				while (UART3Count == 0);
				msg[0] = UART3GetB();
				msg[1] = 35;
				msg[2] = 0;
				UART3PutS(msg);
			}
			else if (op1 == 'V') {		// V+0  - Version code (returns: 22)
				UART3PutS(UCs "22#");
			}
			else if (op1 == 'M') {		// M+0  - Cancel GOTO
				rAim2 = aAlt * GR / 360;
				rAim1 = aAz * GR / 360;
				tFollow = 0;
			}
			else if (op1 == '!') {
				delay(20);
				LcdClr();
				LcdUserLoc(4);
				LcdTLoc(6);
			}
			else if (op1 == '?') {
				UART3PutS(UCs "Radans Tracker ");
				UART3PutS(UCs STVersion);
				UART3PutS(UCs "--------------------------------------------------------------------");
				UART3PutS(UCs "- Custom Commands: ");	// C235000,08022012,+11.0
				UART3PutS(UCs "--------------------------------------------------------------------");
				UART3PutS(UCs "? - Show this help screen");
				UART3PutS(UCs "! - Reset LCD Display");
				UART3PutS(UCs "H - Track to HOME");
				UART3PutS(UCs "Pxxx.xxxEyyy.yyyS       - L+16 - Set Location where x=Long E/W, y=Lat N/S");
				UART3PutS(UCs "Chhmmss,ddmmyyyy,+tz.z  - T+21 - Set Time as Time,Date,Timezone");
				UART3PutS(UCs "Gxx.xxxx,+yy.yyyy       - G+16 - Set Target where x=RA, Y=DEC");
				UART3PutS(UCs "--------------------------------------------------------------------");
				UART3PutS(UCs "- SynScan(tm) AZ Commands: ");
				UART3PutS(UCs "--------------------------------------------------------------------");
				UART3PutS(UCs "BXXXX,YYYY              - B+9  - 16-bit GOTO Az/Alt location where X=Az, Y=Alt.");
				UART3PutS(UCs "bxxxxxxxx,yyyyyyyy      - b+17 - 32-bit GOTO Az-Alt");
				UART3PutS(UCs "RXXXX,YYYY              - R+9  - 16-bit GOTO RA/Dec location where X=RA, Y=Dec.");
				UART3PutS(UCs "rxxxxxxxx,yyyyyyyy      - r+17 - 32-bit GOTO RA-Dec");
				UART3PutS(UCs "E                       - E+0  - 16-bit Get RA/Dec as XXXX,YYYY#");
				UART3PutS(UCs "e                       - e+0  - 32-bit GET RA-Dec");
				UART3PutS(UCs "Tx                      - T+1  - Set Follow where x is 0=off, 1=on");
				UART3PutS(UCs "K                       - K+1  - Echo Command - Kx returns x#");
				UART3PutS(UCs "M                       - M+0  - Cancel GOTO");
				UART3PutS(UCs "V                       - V+0  - SynScan version code (returns: 22)");
				UART3PutS(UCs "--------------------------------------------------------------------");
				UART3PutS(UCs "- TODO Commands:");
				UART3PutS(UCs "--------------------------------------------------------------------");
				UART3PutS(UCs "> Z                     - Z+0  - 16-bit Get Az, Alt as XXXX,YYYY#");
				UART3PutS(UCs "> z                     - z+0  - 32-bit GET Az-Alt as xxxxxxxx,yyyyyyyy#");
				UART3PutS(UCs "> L                     - L+0  - is GOTO in progress yes or no as 1# or 0#");
				UART3PutS(UCs "> J                     - J+0  - is Alignment complete y/n as 1#/0#");
				UART3PutS(UCs "> F+XX                  - F+3  - Move Focus Motor by +99>X>-99 Steps");
				UART3PutS(UCs "--------------------------------------------------------------------");
			}
		}
		// Check for Hard-wired switch commands:
		// PORT 2:0-7 as inputs
		// P2:0   - UP			- Normal mode: tAlt++, Align mode: rotor2-- (will cause up movement)
		// P2:1   - DOWN		- Normal mode: tAlt--, Align mode: rotor2++
		// P2:2   - LEFT		- Normal mode: tAz--, Align mode: rotor1++, Shift: Next Star in Chart
		// P2:8!! - RIGHT		- Normal mode: tAz++, Align mode: rotor1--, Shift: Prev Star in Chart
		// P2:11  - TRACKING	- Toggle Tracking Object or in shift: Current Position
		// P2:5   - HOME		- Track to Home Position on in Shift: Reset to Home
		// P2:6   - FOCUS OUT	- rAim3++ or in Shift: Focus IN rAim3--
		// P2:7   - SHIFT		- Shift Mode
		// TODO: Use P2:11-13 as home switch inputs Az/Alt/Focus respectively from microswitches
		ix = (FIO2PIN0 & 0b11100111) + ((FIO2PIN1 & 1) * 8) + ((FIO2PIN1 & 0b1000) * 2);	// since I killed P2:3-4 remap to P2:8+11!!
		// LcdINumber(ix, 0, 10);

		if (ix > 0) {

			switch(ix) {
			case 1:		// Move/Adjust UP
				if(!tFollow) rAim2 += (GR/1800);
				else rotor2 -= (GR/1800);
				break;
			case 2:		// Move/Adjust DOWN
				if(!tFollow) rAim2 -= (GR/1800);
				else rotor2 += (GR/1800);
				break;
			case 4:		// Move/Adjust LEFT
				if(!tFollow) rAim1 -= (GR/1800);
				else rotor1 += (GR/1800);
				break;
			case 8:		// Move/Adjust RIGHT
				if(!tFollow) rAim1 += (GR/1800);
				else rotor1 -= (GR/1800);
				break;
			case 16:	// Tracking Toggle
				tFollow = 1 - tFollow;
				while (FIO2PIN1 & 0b00001000);
				break;
			case 160:	// RESET Home	(TODO: actual home switches for real home - for now just reset here)
				tHome = 1;		// activate homing
				rotor1 = 0;		// keep going until home switches reset these to 0;
				rotor2 = 0;
				rotor3 = 0;
			case 32:	// GOTO Home
				tFollow = 0;
				rAim1 = 0;
				rAim2 = 0;
				tAz   = 0;
				tAlt  = 0;
				break;
			case 64:	// Focus OUT/+
				rAim3 += 15;
				break;
			case 132:	// Prev Star in Chart
				tChart--;
				if (tChart < 1) tChart = 80;
				tStarName = Chart[tChart];
				tRA = ChartP[tChart][0];
				tDec = ChartP[tChart][1];
				LcdStr(tStarName,5);
				LcdTLoc(6);
				tFollow = 0;
				while (FIO2PIN0 & 0b00000100);
				break;
			case 136:	// Next Star in Chart
				tChart++;
				if (tChart > 80) tChart = 1;
				tStarName = Chart[tChart];
				tRA = ChartP[tChart][0];
				tDec = ChartP[tChart][1];
				LcdStr(tStarName,5);
				LcdTLoc(6);
				tFollow = 0;
				while (FIO2PIN1 & 0b00000001);
				break;
			case 144:	// Tracking from current position
				tAlt = aAlt;
				tAz  = aAz;
				rAim1 = aAz * GR / 360;
				rAim2 = aAlt * GR / 360;
				ConvH2C(0);
				LcdTLoc(6);
				tChart = 0;
				tStarName = Chart[tChart];
				LcdStr(tStarName,5);
				tFollow = 1;
				break;
			case 192:	// Focus IN/-
				rAim3 -= 15;
				break;

			}
		}

		if (tHome == 1) {						// Home switches only reset if in Alignment mode
			if (FIO2PIN1 & 0b00111000) {		// Home Switch indicated
				if (FIO2PIN1 & 0b00001000) {	// Azimuth is at HOME
					rotor1 = 0;
				}
				if (FIO2PIN1 & 0b00010000) {	// Altitude is at HOME
					rotor2 = 0;
				}
				if (FIO2PIN1 & 0b00100000) {	// Focus is at HOME
					rotor3 = 0;
				}
			}
			if ((rotor1+rotor2+rotor3) == 0) {	// if all motors are at home switch position then set to Normal mode
				tHome = 0;
				LcdClr();
				LcdUserLoc(4);
				LcdTLoc(6);
			}
		}
	}
	return 0 ;
}
